@@ -280,6 +280,27 @@ TEST(UtilTest, SubString) {
   EXPECT_EQ(result, "\xe4\xb8\xad\xe9\x87\x8e\xe3\x81\xa7\xe3\x81\x99");
 }
 
+TEST(UtilTest, StartsWith) {
+  const string str = "abcdefg";
+  EXPECT_TRUE(Util::StartsWith(str, ""));
+  EXPECT_TRUE(Util::StartsWith(str, "a"));
+  EXPECT_TRUE(Util::StartsWith(str, "abc"));
+  EXPECT_TRUE(Util::StartsWith(str, "abcdefg"));
+  EXPECT_FALSE(Util::StartsWith(str, "abcdefghi"));
+  EXPECT_FALSE(Util::StartsWith(str, "foobar"));
+}
+
+TEST(UtilTest, EndsWith) {
+  const string str = "abcdefg";
+  EXPECT_TRUE(Util::EndsWith(str, ""));
+  EXPECT_TRUE(Util::EndsWith(str, "g"));
+  EXPECT_TRUE(Util::EndsWith(str, "fg"));
+  EXPECT_TRUE(Util::EndsWith(str, "abcdefg"));
+  EXPECT_FALSE(Util::EndsWith(str, "aaabcdefg"));
+  EXPECT_FALSE(Util::EndsWith(str, "foobar"));
+  EXPECT_FALSE(Util::EndsWith(str, "foobarbuzbuz"));
+}
+
 TEST(UtilTest, StripUTF8BOM) {
   string line;
 
@@ -388,6 +409,94 @@ TEST(UtilTest, SafeStrToUInt64) {
   EXPECT_FALSE(Util::SafeStrToUInt64("", &value));
 }
 
+TEST(UtilTest, SafeStrToDouble) {
+  double value = 1.0;
+
+  EXPECT_TRUE(Util::SafeStrToDouble("0", &value));
+  EXPECT_EQ(0.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble(" \t\r\n\v\f0 \t\r\n\v\f", &value));
+  EXPECT_EQ(0.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble("-0", &value));
+  EXPECT_EQ(0.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble("1.0e1", &value));
+  EXPECT_EQ(10.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble("-5.0e-1", &value));
+  EXPECT_EQ(-0.5, value);
+  EXPECT_TRUE(Util::SafeStrToDouble(".0", &value));
+  EXPECT_EQ(0.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble("0.", &value));
+  EXPECT_EQ(0.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble("0.0", &value));
+  EXPECT_EQ(0.0, value);
+  EXPECT_TRUE(Util::SafeStrToDouble("1.7976931348623158e308", &value));
+  EXPECT_EQ(1.7976931348623158e308, value);  // approximated representation
+                                             // of max of double
+  EXPECT_TRUE(Util::SafeStrToDouble("-1.7976931348623158e308", &value));
+  EXPECT_EQ(-1.7976931348623158e308, value);
+  // GCC accepts hexadecimal number, but VisualC++ does not.
+#ifndef _MSC_VER
+  EXPECT_TRUE(Util::SafeStrToDouble("0x1234", &value));
+  EXPECT_EQ(static_cast<double>(0x1234), value);
+#endif
+
+  EXPECT_FALSE(Util::SafeStrToDouble("1.7976931348623159e308",  // overflow
+                                     &value));
+  EXPECT_FALSE(Util::SafeStrToDouble("-1.7976931348623159e308", &value));
+  EXPECT_FALSE(Util::SafeStrToDouble("3e", &value));
+  EXPECT_FALSE(Util::SafeStrToDouble(".", &value));
+  EXPECT_FALSE(Util::SafeStrToDouble("", &value));
+#ifdef _MSC_VER
+  EXPECT_FALSE(Util::SafeStrToDouble("0x1234", &value));
+#endif
+}
+
+TEST(UtilTest, StringPrintf) {
+  // strings
+  // NB: GCC warns an empty format string, so uses string().c_str() instead.
+  EXPECT_EQ("", Util::StringPrintf(string().c_str()));
+  EXPECT_EQ("", Util::StringPrintf("%s", ""));
+  EXPECT_EQ("hello, world", Util::StringPrintf("hello, world"));
+  EXPECT_EQ("hello, world", Util::StringPrintf("%s", "hello, world"));
+  EXPECT_EQ("hello, world", Util::StringPrintf("%s, %s", "hello", "world"));
+  const char kHello[] = "\xE3\x81\xAF\xE3\x82\x8D\xE3\x83\xBC"  // はろー
+                        "\xE4\xB8\x96\xE7\x95\x8C";  // 世界
+  EXPECT_EQ(kHello, Util::StringPrintf("%s", kHello));
+
+  // 32-bit integers
+  EXPECT_EQ("-2147483648", Util::StringPrintf("%d", kint32min));
+  EXPECT_EQ("2147483647", Util::StringPrintf("%d", kint32max));
+  EXPECT_EQ("4294967295", Util::StringPrintf("%u", kuint32max));
+  EXPECT_EQ("80000000", Util::StringPrintf("%x", kint32min));
+  EXPECT_EQ("7fffffff", Util::StringPrintf("%x", kint32max));
+  EXPECT_EQ("FFFFFFFF", Util::StringPrintf("%X", kuint32max));
+
+  // 64-bit integers
+  EXPECT_EQ("-9223372036854775808",
+            Util::StringPrintf("%"GG_LL_FORMAT"d", kint64min));
+  EXPECT_EQ("9223372036854775807",
+            Util::StringPrintf("%"GG_LL_FORMAT"d", kint64max));
+  EXPECT_EQ("18446744073709551615",
+            Util::StringPrintf("%"GG_LL_FORMAT"u", kuint64max));
+  EXPECT_EQ("8000000000000000",
+            Util::StringPrintf("%"GG_LL_FORMAT"x", kint64min));
+  EXPECT_EQ("7fffffffffffffff",
+            Util::StringPrintf("%"GG_LL_FORMAT"x", kint64max));
+  EXPECT_EQ("FFFFFFFFFFFFFFFF",
+            Util::StringPrintf("%"GG_LL_FORMAT"X", kuint64max));
+
+  // Simple test for floating point numbers
+  EXPECT_EQ("-1.75", Util::StringPrintf("%.2f", -1.75));
+
+  // 4096 is greater than a temporary buffer size (1024 bytes)
+  // which is used in StringPrintf().
+  const string kLongStrA(4096, '.');
+  const string kLongStrB(4096, '_');
+  const string& result = Util::StringPrintf("%s\t%s\n",
+                                            kLongStrA.c_str(),
+                                            kLongStrB.c_str());
+  EXPECT_EQ(kLongStrA + "\t" + kLongStrB + "\n", result);
+}
+
 TEST(UtilTest, HiraganaToKatakana) {
   {
     // "わたしのなまえはなかのですうまーよろしゅう"
@@ -551,6 +660,50 @@ TEST(UtilTest, FullWidthAndHalfWidth) {
   CHECK_EQ("\x20\xe3\x80\x80", output);
 }
 
+TEST(UtilTest, BracketTest) {
+  static const struct BracketType {
+    const char *open_bracket;
+    const char *close_bracket;
+  } kBracketType[] = {
+    //  { "（", "）" },
+    //  { "〔", "〕" },
+    //  { "［", "］" },
+    //  { "｛", "｝" },
+    //  { "〈", "〉" },
+    //  { "《", "》" },
+    //  { "「", "」" },
+    //  { "『", "』" },
+    //  { "【", "】" },
+    //  { "〘", "〙" },
+    //  { "〚", "〛" },
+    { "\xEF\xBC\x88", "\xEF\xBC\x89" },
+    { "\xE3\x80\x94", "\xE3\x80\x95" },
+    { "\xEF\xBC\xBB", "\xEF\xBC\xBD" },
+    { "\xEF\xBD\x9B", "\xEF\xBD\x9D" },
+    { "\xE3\x80\x88", "\xE3\x80\x89" },
+    { "\xE3\x80\x8A", "\xE3\x80\x8B" },
+    { "\xE3\x80\x8C", "\xE3\x80\x8D" },
+    { "\xE3\x80\x8E", "\xE3\x80\x8F" },
+    { "\xE3\x80\x90", "\xE3\x80\x91" },
+    { "\xe3\x80\x98", "\xe3\x80\x99" },
+    { "\xe3\x80\x9a", "\xe3\x80\x9b" },
+    { NULL, NULL },  // sentinel
+  };
+
+  string pair;
+  for (size_t i = 0;
+       (kBracketType[i].open_bracket != NULL ||
+        kBracketType[i].close_bracket != NULL);
+       ++i) {
+    EXPECT_TRUE(Util::IsOpenBracket(kBracketType[i].open_bracket, &pair));
+    EXPECT_EQ(kBracketType[i].close_bracket, pair);
+    EXPECT_TRUE(Util::IsCloseBracket(kBracketType[i].close_bracket, &pair));
+    EXPECT_EQ(kBracketType[i].open_bracket, pair);
+    EXPECT_FALSE(Util::IsOpenBracket(kBracketType[i].close_bracket, &pair));
+    EXPECT_FALSE(Util::IsCloseBracket(kBracketType[i].open_bracket, &pair));
+  }
+}
+
 TEST(UtilTest, KanjiNumberToArabicNumber) {
   {
     // "十"
@@ -1405,6 +1558,90 @@ TEST(UtilTest, ScriptType) {
 }
 
 
+TEST(UtilTest, ScriptTypeWithoutSymbols) {
+  // "くど う"
+  EXPECT_EQ(Util::HIRAGANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x81\x8f\xe3\x81\xa9 \xe3\x81\x86"));
+  // "京 都"
+  EXPECT_EQ(Util::KANJI, Util::GetScriptTypeWithoutSymbols(
+      "\xe4\xba\xac \xe9\x83\xbd"));
+  // "モズ ク"
+  EXPECT_EQ(Util::KATAKANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x83\xa2\xe3\x82\xba\xe3\x82\xaf"));
+  // "モズ クﾓｽﾞｸ"
+  EXPECT_EQ(Util::KATAKANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x83\xa2\xe3\x82\xba \xe3\x82\xaf\xef\xbe\x93\xef\xbd"
+      "\xbd\xef\xbe\x9e\xef\xbd\xb8"));
+  // "Google Earth"
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols(
+      "Google Earth"));
+  // "Google "
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols(
+      "Google "));
+  // " Google"
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols(
+      " Google"));
+  // " Google "
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols(
+      " Google "));
+  // "     g"
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols(
+      "     g"));
+  // ""
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      ""));
+  // " "
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      " "));
+  // "  "
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "   "));
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols("Hello!"));
+  // "Hello!あ"
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "\x48\x65\x6c\x6c\x6f\x21\xe3\x81\x82"));
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols("CD-ROM"));
+  // "CD-ROMア"
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "\x43\x44\x2d\x52\x4f\x4d\xe3\x82\xa2"));
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols("-"));
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols("-A"));
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols("--A"));
+  EXPECT_EQ(Util::ALPHABET, Util::GetScriptTypeWithoutSymbols("--A---"));
+  // "--A-ｱ-"
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "\x2d\x2d\x41\x2d\xef\xbd\xb1\x2d"));
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols("!"));
+  // "・あ"
+  EXPECT_EQ(Util::HIRAGANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x83\xbb\xe3\x81\x82"));
+  // "・・あ"
+  EXPECT_EQ(Util::HIRAGANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x83\xbb\xe3\x83\xbb\xe3\x81\x82"));
+  // "コギト・エルゴ・スム"
+  EXPECT_EQ(Util::KATAKANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x82\xb3\xe3\x82\xae\xe3\x83\x88\xe3\x83\xbb\xe3\x82\xa8"
+      "\xe3\x83\xab\xe3\x82\xb4\xe3\x83\xbb\xe3\x82\xb9\xe3\x83\xa0"));
+  // "コギト・エルゴ・住む"
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x82\xb3\xe3\x82\xae\xe3\x83\x88\xe3\x83\xbb\xe3\x82\xa8"
+      "\xe3\x83\xab\xe3\x82\xb4\xe3\x83\xbb\xe4\xbd\x8f\xe3\x82\x80"));
+  // "人☆名"
+  EXPECT_EQ(Util::KANJI, Util::GetScriptTypeWithoutSymbols(
+      "\xe4\xba\xba\xe2\x98\x86\xe5\x90\x8d"));
+  // "ひとの☆なまえ"
+  EXPECT_EQ(Util::HIRAGANA, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x81\xb2\xe3\x81\xa8\xe3\x81\xae\xe2\x98\x86\xe3\x81\xaa"
+      "\xe3\x81\xbe\xe3\x81\x88"));
+  // "超☆最高です"
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "\xe8\xb6\x85\xe2\x98\x86\xe6\x9c\x80\xe9\xab\x98\xe3\x81\xa7"
+      "\xe3\x81\x99"));
+  // "・--☆"
+  EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptTypeWithoutSymbols(
+      "\xe3\x83\xbb\x2d\x2d\xe2\x98\x86"));
+}
+
 TEST(UtilTest, FormType) {
   // "くどう"
   EXPECT_EQ(Util::FULL_WIDTH, Util::GetFormType("\xe3\x81\x8f\xe3\x81\xa9\xe3"
@@ -1588,6 +1825,17 @@ TEST(UtilTest, IsWindowsX64Test) {
   Util::IsWindowsX64();
 }
 
+TEST(UtilTest, SetIsWindowsX64ModeForTest) {
+  Util::SetIsWindowsX64ModeForTest(Util::IS_WINDOWS_X64_EMULATE_64BIT_MACHINE);
+  EXPECT_TRUE(Util::IsWindowsX64());
+
+  Util::SetIsWindowsX64ModeForTest(Util::IS_WINDOWS_X64_EMULATE_32BIT_MACHINE);
+  EXPECT_FALSE(Util::IsWindowsX64());
+
+  // Clear the emulation.
+  Util::SetIsWindowsX64ModeForTest(Util::IS_WINDOWS_X64_DEFAULT_MODE);
+}
+
 TEST(UtilTest, GetFileVersion) {
   const wchar_t kDllName[] = L"kernel32.dll";
 
@@ -1682,15 +1930,40 @@ TEST(UtilTest, AtomicRename) {
   Util::Unlink(to);
 }
 
+TEST(UtilTest, IsKanaSymbolContained) {
+  const string kFullstop("\xe3\x80\x82"); // "。"
+  const string kSpace(" ");
+  EXPECT_TRUE(Util::IsKanaSymbolContained(kFullstop));
+  EXPECT_TRUE(Util::IsKanaSymbolContained(kSpace + kFullstop));
+  EXPECT_TRUE(Util::IsKanaSymbolContained(kFullstop + kSpace));
+  EXPECT_FALSE(Util::IsKanaSymbolContained(kSpace));
+  EXPECT_FALSE(Util::IsKanaSymbolContained(""));
+}
+
 TEST(UtilTest, Issue2190350) {
   string result = "";
-  Util::UTF8ToSJIS("a", &result);
-  EXPECT_EQ(1, result.length());
-  EXPECT_EQ("a", result);
+  // \xE3\x81\x82 == Hiragana a in UTF8
+  Util::UTF8ToSJIS("\xE3\x81\x82", &result);
+  EXPECT_EQ(2, result.length());
+  // \x82\xA0 == Hiragana a in Shift-JIS
+  EXPECT_EQ("\x82\xA0", result);
 
   result = "";
-  Util::SJISToUTF8("a", &result);
-  EXPECT_EQ(1, result.length());
-  EXPECT_EQ("a", result);
+  Util::SJISToUTF8("\x82\xA0", &result);
+  EXPECT_EQ(3, result.length());
+  EXPECT_EQ("\xE3\x81\x82", result);
 }
+
+TEST(UtilTest, Fingerprint32WithSeed_uint32) {
+  const uint32 seed = 0xabcdef;
+
+  const uint32 num = 0x12345678;    // Assumed little endian
+  const uint32 num_hash = Util::Fingerprint32WithSeed(num, seed);
+
+  const char* str = "\x78\x56\x34\x12";
+  const uint32 str_hash = Util::Fingerprint32WithSeed(str, 4, seed);
+
+  EXPECT_EQ(num_hash, str_hash) << num_hash << " != " << str_hash;
+}
+
 }  // namespace mozc
