@@ -42,6 +42,7 @@
 
 namespace mozc {
 namespace commands {
+class Context;
 class KeyEvent;
 }  // namespace commands
 
@@ -57,6 +58,12 @@ class Composer {
   // Reset all composing data except table.
   void Reset();
 
+  // Reset input mode.  When the current input mode is
+  // HalfAlphanumeric by pressing capital alphabet, this function
+  // revert the input mode from HalfAlphanumeric to the previous input
+  // mode.
+  void ResetInputMode();
+
   // Reload the configuration.
   void ReloadConfig();
 
@@ -75,6 +82,8 @@ class Composer {
 
   // Update the input mode considering the input modes of the
   // surrounding characters.
+  // If the input mode should not be changed based on the surrounding text,
+  // do not call this method (e.g. CursorToEnd, CursorToBeginning).
   void UpdateInputMode();
 
   transliteration::TransliterationType GetInputMode() const;
@@ -105,10 +114,13 @@ class Composer {
 
   void InsertCharacterAt(size_t pos, const string &input);
   void DeleteAt(size_t pos);
-  void BackspaceAt(size_t pos);
   void InsertCharacter(const string &input);
   void InsertCharacterPreedit(const string &input);
   void InsertCharacterKeyAndPreedit(const string &key, const string &preedit);
+  void InsertCharacterPreeditAt(size_t pos, const string &input);
+  void InsertCharacterKeyAndPreeditAt(size_t pos,
+                                      const string &key,
+                                      const string &preedit);
   bool InsertCharacterKeyEvent(const commands::KeyEvent &key);
   void Delete();
   void Backspace();
@@ -142,6 +154,12 @@ class Composer {
   // status and user's settings.
   void AutoSwitchMode();
 
+  // Insert preceding text if |context| meets some condition.  It returns the
+  // number of characters at the end of context.preceding_text() which are
+  // involved to composition and must be deleted from preceding text.
+  // NOTE: Do not call this function if client cannot delete preceding text.
+  size_t InsertPrecedingText(const commands::Context &context);
+
   static Composer *Create(const Table *table);
   // Transform characters for preferred number format.  If any
   // characters are transformed true is returned.
