@@ -121,6 +121,14 @@ TEST(CharChunkTest, AddInput_ForN) {
   EXPECT_EQ("", chunk1.conversion());
   EXPECT_EQ("ny", chunk1.pending());
   EXPECT_EQ("n", input);
+
+  input = "a";
+  EXPECT_FALSE(chunk1.AddInputInternal(table, &input));
+  EXPECT_TRUE(chunk1.IsFixed());
+  EXPECT_EQ("nya", chunk1.raw());
+  EXPECT_EQ("[NYA]", chunk1.conversion());
+  EXPECT_EQ("", chunk1.pending());
+  EXPECT_EQ("", input);
 }
 
 TEST(CharChunkTest, AddInput_WithString) {
@@ -464,6 +472,75 @@ TEST(CharChunkTest, CaseSensitive) {
   EXPECT_TRUE(chunk.pending().empty());
 }
 
+TEST(CharChunkTest, AlphanumericOfSSH) {
+  // This is a unittest against http://b/3199626
+  // 'ssh' (っｓｈ) + F10 should be 'ssh'.
+  const TransliteratorInterface *kHiraganaT12r =
+      TransliteratorsJa::GetHiraganaTransliterator();
+  Table table;
+  // "っ"
+  table.AddRule("ss", "\xE3\x81\xA3", "s");
+  // "し"
+  table.AddRule("shi", "\xE3\x81\x97", "");
+
+  CharChunk chunk;
+  chunk.SetTransliterator(kHiraganaT12r);
+
+  {
+    string input("ssh");
+    chunk.AddInput(table, &input);
+    EXPECT_TRUE(input.empty());
+    EXPECT_EQ("ssh", chunk.raw());
+    // "っ"
+    EXPECT_EQ("\xE3\x81\xA3", chunk.conversion());
+    EXPECT_EQ("sh", chunk.pending());
+    // empty() is intentionaly not used due to check the actual value.
+    EXPECT_EQ("", chunk.ambiguous());
+  }
+  {
+    string result;
+    chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+    // "っｓｈ"
+    EXPECT_EQ("\xE3\x81\xA3\xEF\xBD\x93\xEF\xBD\x88", result);
+  }
+
+  // Break down of the internal procedures
+  chunk.Clear();
+  chunk.SetTransliterator(kHiraganaT12r);
+  {
+    string input("s");
+    chunk.AddInput(table, &input);
+    EXPECT_TRUE(input.empty());
+    EXPECT_EQ("s", chunk.raw());
+    // empty() is intentionaly not used due to check the actual value.
+    EXPECT_EQ("", chunk.conversion());
+    EXPECT_EQ("s", chunk.pending());
+    EXPECT_EQ("", chunk.ambiguous());
+  }
+  {
+    string input("s");
+    chunk.AddInput(table, &input);
+    EXPECT_TRUE(input.empty());
+    EXPECT_EQ("ss", chunk.raw());
+    // "っ"
+    EXPECT_EQ("\xE3\x81\xA3", chunk.conversion());
+    EXPECT_EQ("s", chunk.pending());
+    // empty() is intentionaly not used due to check the actual value.
+    EXPECT_EQ("", chunk.ambiguous());
+  }
+  {
+    string input("h");
+    chunk.AddInput(table, &input);
+    EXPECT_TRUE(input.empty());
+    EXPECT_EQ("ssh", chunk.raw());
+    // "っ"
+    EXPECT_EQ("\xE3\x81\xA3", chunk.conversion());
+    EXPECT_EQ("sh", chunk.pending());
+    // empty() is intentionaly not used due to check the actual value.
+    EXPECT_EQ("", chunk.ambiguous());
+  }
+}
+
 TEST(CharChunkTest, Issue2190364) {
   // This is a unittest against http://b/2190364
   Table table;
@@ -532,6 +609,310 @@ TEST(CharChunkTest, Issue2209634) {
   EXPECT_EQ("q@", output);
 }
 
+TEST(CharChunkTest, Issue2819580) {
+  // This is an unittest against http://b/2819580.
+  // 'y' after 'n' disappears.
+
+  const TransliteratorInterface *kHiraganaT12r =
+      TransliteratorsJa::GetHiraganaTransliterator();
+
+  Table table;
+  // "ぽ"
+  table.AddRule("po", "\xe3\x81\xbd", "");
+  // "ん"
+  table.AddRule("n", "\xe3\x82\x93", "");
+  // "な"
+  table.AddRule("na", "\xe3\x81\xaa", "");
+  // "や"
+  table.AddRule("ya", "\xe3\x82\x84", "");
+  // "にゃ"
+  table.AddRule("nya", "\xe3\x81\xab\xe3\x82\x83", "");
+
+  // Test for reported situation ("ny").
+  // AddInput ver.
+  {
+    CharChunk chunk;
+    chunk.SetTransliterator(kHiraganaT12r);
+
+    {
+      string input("n");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("y");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string result;
+      chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+      // "んｙ"
+      EXPECT_EQ("\xe3\x82\x93\xef\xbd\x99", result);
+    }
+
+    {
+      string input("a");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string result;
+      chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+      // "にゃ"
+      EXPECT_EQ("\xe3\x81\xab\xe3\x82\x83", result);
+    }
+  }
+
+  // Test for reported situation (ny).
+  // AddInputAndConvertedChar ver.
+  {
+    CharChunk chunk;
+    chunk.SetTransliterator(kHiraganaT12r);
+
+    {
+      string input("n");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("y");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("a");
+      string converted("a");
+      chunk.AddInputAndConvertedChar(table, &input, &converted);
+    }
+    {
+      string result;
+      chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+      // "にゃ"
+      EXPECT_EQ("\xe3\x81\xab\xe3\x82\x83", result);
+    }
+  }
+
+  // Test for reported situation ("pony").
+  {
+    CharChunk chunk;
+    chunk.SetTransliterator(kHiraganaT12r);
+
+    {
+      string input("p");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("o");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("n");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("y");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string result;
+      chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+      // "ぽんｙ"
+      EXPECT_EQ("\xe3\x81\xbd\xe3\x82\x93\xef\xbd\x99", result);
+    }
+
+    {
+      string input("a");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string result;
+      chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+      // "ぽにゃ"
+      EXPECT_EQ("\xe3\x81\xbd\xe3\x81\xab\xe3\x82\x83", result);
+    }
+  }
+
+  // The first input is not contained in the table.
+  {
+    CharChunk chunk;
+    chunk.SetTransliterator(kHiraganaT12r);
+
+    {
+      string input("z");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("n");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string input("y");
+      chunk.AddInput(table, &input);
+    }
+    {
+      string result;
+      chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+      // "ｚんｙ"
+      EXPECT_EQ("\xef\xbd\x9a\xe3\x82\x93\xef\xbd\x99", result);
+    }
+  }
+}
+
+TEST(CharChunkTest, Issue2990253) {
+  // http://b/2990253
+  // SplitChunk fails.
+  // Ambiguous text is left in rhs CharChunk invalidly.
+
+  const TransliteratorInterface *kHiraganaT12r =
+      TransliteratorsJa::GetHiraganaTransliterator();
+
+  Table table;
+  // "ん"
+  table.AddRule("n", "\xe3\x82\x93", "");
+  // "な"
+  table.AddRule("na", "\xe3\x81\xaa", "");
+  // "や"
+  table.AddRule("ya", "\xe3\x82\x84", "");
+  // "にゃ"
+  table.AddRule("nya", "\xe3\x81\xab\xe3\x82\x83", "");
+
+  CharChunk chunk, left_new_chunk;
+
+  {
+    string input("n");
+    chunk.AddInput(table, &input);
+  }
+  {
+    string input("y");
+    chunk.AddInput(table, &input);
+  }
+  chunk.SplitChunk(kHiraganaT12r, size_t(1), &left_new_chunk);
+  {
+    string result;
+    chunk.AppendFixedResult(table, kHiraganaT12r, &result);
+    // "ｙ"
+    EXPECT_EQ("\xef\xbd\x99", result);
+  }
+}
+
+TEST(CharChunkTest, Combine) {
+  {
+    CharChunk lhs, rhs;
+    lhs.set_ambiguous("LA");
+    lhs.set_conversion("LC");
+    lhs.set_pending("LP");
+    lhs.set_raw("LR");
+    rhs.set_status(CharChunk::NO_RAW);
+
+    rhs.set_ambiguous("RA");
+    rhs.set_conversion("RC");
+    rhs.set_pending("RP");
+    rhs.set_raw("RR");
+    rhs.set_status(CharChunk::NO_CONVERSION);
+
+    rhs.Combine(lhs);
+    EXPECT_EQ("LARA", rhs.ambiguous());
+    EXPECT_EQ("LCRC", rhs.conversion());
+    EXPECT_EQ("LPRP", rhs.pending());
+    EXPECT_EQ("LRRR", rhs.raw());
+    EXPECT_TRUE(rhs.has_status(CharChunk::NO_CONVERSION));
+  }
+
+  {  // lhs' ambigous is empty.
+    CharChunk lhs, rhs;
+    lhs.set_ambiguous("");
+    lhs.set_conversion("LC");
+    lhs.set_pending("LP");
+    lhs.set_raw("LR");
+    rhs.set_status(CharChunk::NO_RAW);
+
+    rhs.set_ambiguous("RA");
+    rhs.set_conversion("RC");
+    rhs.set_pending("RP");
+    rhs.set_raw("RR");
+    rhs.set_status(CharChunk::NO_CONVERSION);
+
+    rhs.Combine(lhs);
+    EXPECT_EQ("", rhs.ambiguous());
+    EXPECT_EQ("LCRC", rhs.conversion());
+    EXPECT_EQ("LPRP", rhs.pending());
+    EXPECT_EQ("LRRR", rhs.raw());
+    EXPECT_TRUE(rhs.has_status(CharChunk::NO_CONVERSION));
+  }
+
+  {  // rhs' ambigous is empty.
+    CharChunk lhs, rhs;
+    lhs.set_ambiguous("LA");
+    lhs.set_conversion("LC");
+    lhs.set_pending("LP");
+    lhs.set_raw("LR");
+    rhs.set_status(CharChunk::NO_RAW);
+
+    rhs.set_ambiguous("");
+    rhs.set_conversion("RC");
+    rhs.set_pending("RP");
+    rhs.set_raw("RR");
+    rhs.set_status(CharChunk::NO_CONVERSION);
+
+    rhs.Combine(lhs);
+    EXPECT_EQ("LARP", rhs.ambiguous());
+    EXPECT_EQ("LCRC", rhs.conversion());
+    EXPECT_EQ("LPRP", rhs.pending());
+    EXPECT_EQ("LRRR", rhs.raw());
+    EXPECT_TRUE(rhs.has_status(CharChunk::NO_CONVERSION));
+  }
+}
+
+TEST(CharChunkTest, IsConvertible) {
+  CharChunk chunk;
+  const TransliteratorInterface *kHiraganaT12r =
+      TransliteratorsJa::GetHiraganaTransliterator();
+  const TransliteratorInterface *kFullAsciiT12r =
+      TransliteratorsJa::GetFullAsciiTransliterator();
+  Table table;
+  // "ん"
+  table.AddRule("n", "\xe3\x82\x93", "");
+  // "な"
+  table.AddRule("na", "\xe3\x81\xaa", "");
+  {
+    // If pending is empty, returns false.
+    chunk.Clear();
+    EXPECT_EQ("", chunk.pending());
+    EXPECT_FALSE(chunk.IsConvertible(kHiraganaT12r, table, "n"));
+  }
+  {
+    // If t12r is inconsistent, returns false.
+    chunk.Clear();
+    chunk.SetTransliterator(kHiraganaT12r);
+    string input("n");
+    chunk.AddInput(table, &input);
+    EXPECT_EQ("n", chunk.pending());
+    EXPECT_FALSE(chunk.IsConvertible(kFullAsciiT12r, table, "a"));
+  }
+  {
+    // If no entries are found from the table, returns false.
+    chunk.Clear();
+    chunk.SetTransliterator(kHiraganaT12r);
+    string input("n");
+    chunk.AddInput(table, &input);
+    EXPECT_EQ("n", chunk.pending());
+    EXPECT_FALSE(chunk.IsConvertible(kHiraganaT12r, table, "x"));
+  }
+  {
+    // If found entry does not consume all of input, returns false.
+    chunk.Clear();
+    chunk.SetTransliterator(kHiraganaT12r);
+    string input("n");
+    chunk.AddInput(table, &input);
+    EXPECT_EQ("n", chunk.pending());
+    EXPECT_FALSE(chunk.IsConvertible(kHiraganaT12r, table, "y"));
+  }
+  {
+    // [pending='n'] + [input='a'] is convertible (single combination).
+    chunk.Clear();
+    chunk.SetTransliterator(kHiraganaT12r);
+    string input("n");
+    chunk.AddInput(table, &input);
+    EXPECT_EQ("n", chunk.pending());
+    EXPECT_TRUE(chunk.IsConvertible(kHiraganaT12r, table, "a"));
+  }
+}
 
 }  // namespace composer
 }  // namespace mozc
