@@ -57,6 +57,8 @@ class ConverterImpl : public ConverterInterface {
 
   bool StartConversion(Segments *segments,
                        const string &key) const;
+  bool StartReverseConversion(Segments *segments,
+                              const string &key) const;
   bool StartPrediction(Segments *segments,
                        const string &key) const;
   bool StartSuggestion(Segments *segments,
@@ -114,15 +116,13 @@ void SetKey(Segments *segments, const string &key) {
   segments->clear_revert_entries();
 
   mozc::Segment *seg = segments->add_segment();
+  DCHECK(seg);
+
   seg->clear();
   seg->set_key(key);
   seg->set_segment_type(mozc::Segment::FREE);
 
-  if (VLOG_IS_ON(2)) {
-    string buf;
-    segments->DebugString(&buf);
-    LOG(INFO) << buf;
-  }
+  VLOG(2) << segments->DebugString();
 }
 
 ConverterInterface *g_converter = NULL;
@@ -161,6 +161,11 @@ bool ConverterImpl::StartConversion(Segments *segments,
   }
   return false;
 }
+bool ConverterImpl::StartReverseConversion(Segments *segments,
+                                           const string &key) const {
+  LOG(WARNING) << "StartReverseConversion is not implemented";
+  return false;
+}
 bool ConverterImpl::StartPrediction(Segments *segments,
                                     const string &key) const {
   SetKey(segments, key);
@@ -557,6 +562,7 @@ void ConverterUtil::InitSegmentsFromString(const string &key,
   c->Init();
   c->value = preedit;
   c->content_value = preedit;
+  c->key = key;
   c->content_key = key;
 }
 }  // namespace mozc
