@@ -43,53 +43,48 @@
 #include "converter/segments.h"
 
 namespace mozc {
-
 namespace {
 
-static const size_t kMaxHistorySize = 32;
-
-class CompareByScore {
- public:
-  bool operator()(mozc::Segment::Candidate *a,
-                  mozc::Segment::Candidate *b) const {
-    return (a->cost < b->cost);
-  }
-};
-
-// Ad-hoc function to detect English candidate
-bool IsKatakanaT13N(const string &value) {
-  for (size_t i = 0; i < value.size(); ++i) {
-    if (value[i] == 0x20 ||
-        (value[i] >= 0x41 && value[i] <= 0x5A) ||
-        (value[i] >= 0x61 && value[i] <= 0x7A)) {
-      // do nothing
-    } else {
-      return false;
-    }
-  }
-  return true;
-}
-}  // namespace
+const size_t kMaxHistorySize = 32;
 
 // "ひらがな"
-static char kHiragana[] = "\xE3\x81\xB2\xE3\x82\x89\xE3\x81\x8C\xE3\x81\xAA";
+const char kHiragana[] = "\xE3\x81\xB2\xE3\x82\x89\xE3\x81\x8C\xE3\x81\xAA";
 // "カタカナ"
-static char kKatakana[] = "\xE3\x82\xAB\xE3\x82\xBF\xE3\x82\xAB\xE3\x83\x8A";
+const char kKatakana[] = "\xE3\x82\xAB\xE3\x82\xBF\xE3\x82\xAB\xE3\x83\x8A";
 // "数字"
-static char kNumber[] = "\xE6\x95\xB0\xE5\xAD\x97";
+const char kNumber[] = "\xE6\x95\xB0\xE5\xAD\x97";
 // "アルファベット"
-static char kAlphabet[] = "\xE3\x82\xA2\xE3\x83\xAB\xE3\x83\x95\xE3\x82\xA1"
+const char kAlphabet[] = "\xE3\x82\xA2\xE3\x83\xAB\xE3\x83\x95\xE3\x82\xA1"
                           "\xE3\x83\x99\xE3\x83\x83\xE3\x83\x88";
 // "漢字"
-static char kKanji[] = "\xe6\xbc\xa2\xe5\xad\x97";
+const char kKanji[] = "\xe6\xbc\xa2\xe5\xad\x97";
 
+// "[全]"
+const char kFullWidth[] = "[\xE5\x85\xA8]";
+
+// "[半]"
+const char kHalfWidth[] = "[\xE5\x8D\x8A]";
+
+// "<機種依存文字>"
+const char kPlatformDependent[] = "<\xE6\xA9\x9F\xE7\xA8\xAE"
+    "\xE4\xBE\x9D\xE5\xAD\x98\xE6\x96\x87\xE5\xAD\x97>";
+
+// "<もしかして>"
+const char kDidYouMean[] =
+    "<\xE3\x82\x82\xE3\x81\x97\xE3\x81\x8B\xE3\x81\x97\xE3\x81\xA6>";
+
+bool IsKatakanaT13N(const Segment::Candidate &candidate) {
+  return (Util::GetScriptType(candidate.content_key) == Util::HIRAGANA &&
+          Segment::IsKatakanaT13NValue(candidate.content_value));
+}
+}  // namespace
 
 void Segment::Candidate::SetDefaultDescription(int description_type) {
   string message;
 
   if (description_type & Segment::Candidate::CHARACTER_FORM) {
     const string &value = this->value;
-    const Util::ScriptType type = Util::GetScriptType(value);
+    const Util::ScriptType type = Util::GetScriptTypeWithoutSymbols(value);
     switch (type) {
       case Util::HIRAGANA:
         message = kHiragana;
@@ -181,15 +176,21 @@ void Segment::Candidate::ResetDescription(int description_type) {
     switch (form) {
       case Util::FULL_WIDTH:
         // description = "[全]";
-        description = "[\xE5\x85\xA8]";
+        description = kFullWidth;
         break;
       case Util::HALF_WIDTH:
         // description = "[半]";
-        description = "[\xE5\x8D\x8A]";
+        description = kHalfWidth;
       break;
     default:
       break;
     }
+  } else if (description_type & Segment::Candidate::FULL_WIDTH) {
+    // description = "[全]";
+    description = kFullWidth;
+  } else if (description_type & Segment::Candidate::HALF_WIDTH) {
+    // description = "[半]";
+    description = kHalfWidth;
   }
 
   // add main message
@@ -207,9 +208,8 @@ void Segment::Candidate::ResetDescription(int description_type) {
       if (!description.empty()) {
         description += " ";
       }
-      //      description += "<機種依存文字>";
-      description += "<\xE6\xA9\x9F\xE7\xA8\xAE"
-          "\xE4\xBE\x9D\xE5\xAD\x98\xE6\x96\x87\xE5\xAD\x97>";
+      // description += "<機種依存文字>";
+      description += kPlatformDependent;
     }
   }
 
@@ -218,16 +218,37 @@ void Segment::Candidate::ResetDescription(int description_type) {
       POSMatcher::IsZipcode(lid) && lid == rid) {
     description = content_key;
   }
+
+  // Spelling Correction description
+  // Delete all messages here.
+  if (is_spelling_correction) {
+    description = kDidYouMean;
+  }
+}
+
+string Segment::Candidate::functional_key() const {
+  if (key.size() <= content_key.size()) {
+    return "";
+  }
+  return key.substr(content_key.size(),
+                    key.size() - content_key.size());
+}
+
+string Segment::Candidate::functional_value() const {
+  if (value.size() <= content_value.size()) {
+    return "";
+  }
+  return value.substr(content_value.size(),
+                      value.size() - content_value.size());
 }
 
 Segment::Segment()
     : segment_type_(FREE),
       requested_candidates_size_(0),
-      nbest_generator_(new NBestGenerator),
-      pool_(new ObjectPool<Candidate>(16)),
-      is_numbers_expanded_(false),
       initialized_transliterations_(false),
-      katakana_t13n_length_(0) {}
+      all_expanded_(false),
+      nbest_generator_(new NBestGenerator),
+      pool_(new ObjectPool<Candidate>(16)) {}
 
 Segment::~Segment() {}
 
@@ -415,8 +436,7 @@ void Segment::clear_candidates() {
   pool_->Free();
   candidates_.clear();
   requested_candidates_size_ = 0;
-  katakana_t13n_length_ = 0;
-  is_numbers_expanded_ = false;
+  all_expanded_ = false;
 }
 
 Segment::Candidate *Segment::push_back_candidate() {
@@ -530,22 +550,6 @@ void Segment::clear() {
   nbest_generator_->Reset();
 }
 
-bool Segment::GetCandidates(size_t size) {
-  if (size > candidates_.size()) {
-    return Expand(size - candidates_.size());
-  }
-  return true;
-}
-
-bool Segment::has_candidate_value(const string &value) const {
-  for (size_t i = 0; i < candidates_.size(); ++i) {
-    if (candidates_[i]->value == value) {
-      return true;
-    }
-  }
-  return false;
-}
-
 bool Segment::ExpandAlternative(int i) {
   if (i < 0 || i >= candidates_size()) {
     LOG(WARNING) << "invalid index";
@@ -553,10 +557,7 @@ bool Segment::ExpandAlternative(int i) {
   }
 
   Candidate *org_c = mutable_candidate(i);
-  if (org_c == NULL) {
-    LOG(WARNING) << "org_c is NULL";
-    return false;
-  }
+  DCHECK(org_c);
 
   if (!org_c->can_expand_alternative) {
     VLOG(1) << "Canidate has NO_NORMALIZATION node";
@@ -583,38 +584,120 @@ bool Segment::ExpandAlternative(int i) {
     alternative_content_value = alternative_value;
   }
 
-  Candidate *new_c = insert_candidate(i);
-  if (new_c == NULL) {
-    LOG(WARNING) << "new_c is NULL";
-    return false;
+  CharacterFormManager::FormType default_form
+      = CharacterFormManager::UNKNOWN_FORM;
+  CharacterFormManager::FormType alternative_form
+      = CharacterFormManager::UNKNOWN_FORM;
+
+  int default_description_type =
+      (Segment::Candidate::CHARACTER_FORM |
+       Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER);
+
+  int alternative_description_type =
+      (Segment::Candidate::CHARACTER_FORM |
+       Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER);
+
+  if (CharacterFormManager::GetFormTypesFromStringPair(
+          default_value,
+          &default_form,
+          alternative_value,
+          &alternative_form)) {
+    if (default_form == CharacterFormManager::HALF_WIDTH) {
+      default_description_type |= Segment::Candidate::HALF_WIDTH;
+    } else if (default_form == CharacterFormManager::FULL_WIDTH) {
+      default_description_type |= Segment::Candidate::FULL_WIDTH;
+    }
+    if (alternative_form == CharacterFormManager::HALF_WIDTH) {
+      alternative_description_type |= Segment::Candidate::HALF_WIDTH;
+    } else if (alternative_form == CharacterFormManager::FULL_WIDTH) {
+      alternative_description_type |= Segment::Candidate::FULL_WIDTH;
+    }
+  } else {
+    default_description_type     |= Segment::Candidate::FULL_HALF_WIDTH;
+    alternative_description_type |= Segment::Candidate::FULL_HALF_WIDTH;
   }
 
+  Candidate *new_c = insert_candidate(i);
+  DCHECK(new_c);
+
   new_c->Init();
+  new_c->key = org_c->key;
+  new_c->value = default_value;
   new_c->content_key = org_c->content_key;
+  new_c->content_value = default_content_value;
   new_c->cost = org_c->cost;
   new_c->structure_cost = org_c->structure_cost;
   new_c->lid = org_c->lid;
   new_c->rid = org_c->rid;
-  new_c->value = default_value;
-  new_c->content_value = default_content_value;
-  new_c->SetDefaultDescription(
-      Segment::Candidate::FULL_HALF_WIDTH |
-      Segment::Candidate::CHARACTER_FORM |
-      Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER);
+  new_c->SetDefaultDescription(default_description_type);
 
   org_c->value = alternative_value;
   org_c->content_value = alternative_content_value;
   org_c->description.clear();
-  org_c->SetDefaultDescription(
-      Segment::Candidate::FULL_HALF_WIDTH |
-      Segment::Candidate::CHARACTER_FORM |
-      Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER);
+  org_c->SetDefaultDescription(alternative_description_type);
 
   return true;
 }
 
-bool Segment::Expand(size_t size) {
-  if (size <= 0) {
+// static
+bool Segment::ExpandEnglishVariants(const string &input,
+                                    vector<string> *variants) {
+  if (input.empty()) {
+    return false;
+  }
+
+  // multi-word
+  if (input.find(" ") != string::npos) {
+    return false;
+  }
+
+  string lower = input;
+  string upper = input;
+  string capitalized = input;
+  Util::LowerString(&lower);
+  Util::UpperString(&upper);
+  Util::CapitalizeString(&capitalized);
+
+  if (lower == upper) {
+    // given word is non-ascii.
+    return false;
+  }
+
+  variants->clear();
+  if (input != lower && input != upper && input != capitalized) {
+    variants->push_back(lower);
+    return true;
+  }
+
+  if (input != lower) {
+    variants->push_back(lower);
+  }
+  if (input != capitalized) {
+    variants->push_back(capitalized);
+  }
+  if (input != upper) {
+    variants->push_back(upper);
+  }
+
+  return true;
+}
+
+bool Segment::IsKatakanaT13NValue(const string &value) {
+  for (size_t i = 0; i < value.size(); ++i) {
+    if (value[i] == 0x20 || value[i] == 0x21 || value[i] == 0x2D ||
+        // " ", "!", "-"
+        (value[i] >= 0x41 && value[i] <= 0x5A) ||  // A..Z
+        (value[i] >= 0x61 && value[i] <= 0x7A)) {  // a..z
+      // do nothing
+    } else {
+      return false;
+    }
+  }
+  return true;
+}
+
+bool Segment::GetCandidates(size_t target_size) {
+  if (target_size <= 0) {
     LOG(WARNING) << "invalid size";
     return false;
   }
@@ -624,7 +707,11 @@ bool Segment::Expand(size_t size) {
     return false;
   }
 
-  const size_t target_size = candidates_size() + size;
+  // already expanded
+  if (all_expanded_) {
+    return true;
+  }
+
   if (requested_candidates_size() >= target_size) {
     // Avaliable candidates have been already generated.
     return true;
@@ -633,139 +720,51 @@ bool Segment::Expand(size_t size) {
 
   // if NBestGenerator::Next() returns NULL,
   // no more entries are generated.
-  bool all_expanded = false;
-
   while (candidates_size() < target_size) {
     Candidate *c = push_back_candidate();
-    const Node *begin_node = NULL;
-    const Node *end_node = NULL;
-    if (c == NULL) {
-      LOG(WARNING) << "candidate is NULL";
-      return false;
-    }
+    DCHECK(c);
     c->Init();
-    if (!nbest_generator_->Next(c, &begin_node, &end_node)) {
+
+    if (!nbest_generator_->Next(c)) {
       pop_back_candidate();
-      all_expanded = true;   // no more entries
+      // set all_expanded_ to be true that
+      // GetCandidates() is never called again.
+      // http://b/issue?id=2868423
+      all_expanded_ = true;   // no more entries
       break;
     }
+
     c->SetDefaultDescription(
         Segment::Candidate::FULL_HALF_WIDTH |
         Segment::Candidate::CHARACTER_FORM |
         Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER |
         Segment::Candidate::ZIPCODE);
 
-    bool remove_candidate = false;
-
-    // Check all nodes are DEFAULT_NORMALIZATION
-    // Otherwise, don't call ExpandAlternative()
-    for (const Node *node = begin_node;
-         node != end_node;
-         node = node->next) {
-      const bool is_katakana_t13n = IsKatakanaT13N(node->value);
-      // Hack for Katakana 13n.
-      // Suppose we have two katakana's  (A and B) in candidate results
-      // A is t13ned into A',A'' ..
-      // A is t13ned into B',B'' ..
-      // Here we don't show B' B'' ...
-      // in the candidate as they are noisy -- e.g.,
-      // partially matching to the entier candidate string.
-      // TODO(taku): remove it when we can improve candidate filter
-      if (is_katakana_t13n) {
-        if (node != begin_node ||   // only allow prefix match
-            (katakana_t13n_length_ != 0 &&   // different candidate
-             Util::GetScriptType(node->key) == Util::HIRAGANA &&
-             node->key.size() != katakana_t13n_length_)) {
-          remove_candidate = true;
-          break;
+    // Make KatakanaT13n Candidates
+    if (IsKatakanaT13N(*c)) {
+      c->can_expand_alternative = false;
+      vector<string> variants;
+      // TODO(taku): could be possible to move this logic to Rewriter.
+      if (Segment::ExpandEnglishVariants(c->content_value, &variants)) {
+        for (size_t i = 0; i < variants.size(); ++i) {
+          Candidate *variant_c = add_candidate();
+          DCHECK(variant_c);
+          variant_c->Init();
+          variant_c->value = variants[i] + c->functional_value();
+          variant_c->key = c->key;
+          variant_c->content_value = variants[i];
+          variant_c->content_key = c->content_key;
+          variant_c->cost = c->cost + 1;
+          variant_c->structure_cost = c->structure_cost + 1;
+          variant_c->lid = c->lid;
+          variant_c->rid = c->rid;
+          variant_c->can_expand_alternative = false;
+          variant_c->SetDefaultDescription(
+              Segment::Candidate::FULL_HALF_WIDTH |
+              Segment::Candidate::CHARACTER_FORM |
+              Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER);
         }
-        katakana_t13n_length_ = node->key.size();
       }
-
-      if (node->normalization_type == Node::NO_NORMALIZATION ||
-          is_katakana_t13n) {
-        c->can_expand_alternative = false;
-        break;
-      }
-    }
-
-    if (remove_candidate) {
-      VLOG(1) << c->value << " is remvoed";
-      pop_back_candidate();
-      continue;
-    }
-
-    // Make Arabic number node:
-    if (!is_numbers_expanded_ &&
-        POSMatcher::IsNumber(c->lid) && c->lid == c->rid) {
-      string new_value, new_content_value, new_content_key;
-      bool content = false;
-      const Node *node = NULL;
-      // Current mozc doesn't have a functionality to classify
-      // a word into functional-word or content-word.
-      // We simply assume that the first word inside a segmenet
-      // is content word,
-      // e.g, "知らないことはない" => "知ら" is content, and "ないことはない"
-      // s funcitonal. However, that won't work for numbers. "三百円"
-      // --"三" is content, "百円" is functional
-      // with the current implementation.
-      // This for loop just finds a sequence of numbers and rewrite
-      // the content_value. Ideally, we should do this inside
-      // nbest_generator_.cc after implementing
-      // content-word/functional-word classifier
-      for (node = begin_node; node != end_node; node = node->next) {
-        if (!content && POSMatcher::IsNumber(node->lid) &&
-            node->lid == node->rid) {
-          new_content_key += node->key;
-          new_content_value += node->value;
-        } else {
-          content = true;
-        }
-        new_value += node->value;
-      }
-
-      // rewrite content_value
-      c->value = new_value;
-      c->content_value = new_content_value;
-      c->content_key = new_content_key;
-
-      // Expand Full/Half
-      ExpandAlternative(candidates_size() - 1);
-
-      // We respect the best candidate ImmutableConverter generates.
-      // Number rewriter expands candidates if candidate contains
-      // arabic number.
-      // Example:
-      // key: "20まんえん"
-      // best: "20万円"
-      // arabic alternative: "200000円"
-      // If the best candidate is already arabic number,
-      // no need to generate alternatives
-      string kanji_number, arabic_number, half_width_new_content_value;
-      Util::FullWidthToHalfWidth(new_content_key,
-                                 &half_width_new_content_value);
-      if (Util::NormalizeNumbers(new_content_value, true,
-                                 &kanji_number, &arabic_number) &&
-          arabic_number != half_width_new_content_value) {
-        Candidate *arabic_c = push_back_candidate();
-        const string suffix =
-            new_value.substr(new_content_value.size(),
-                             new_value.size() - new_content_value.size());
-        arabic_c->Init();
-        arabic_c->value = arabic_number + suffix;
-        arabic_c->content_value = arabic_number;
-        arabic_c->content_key = c->content_key;
-        arabic_c->cost = c->cost;
-        arabic_c->structure_cost = c->structure_cost;
-        arabic_c->lid = c->lid;
-        arabic_c->rid = c->rid;
-        arabic_c->SetDefaultDescription(
-            Segment::Candidate::CHARACTER_FORM |
-            Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER);
-        ExpandAlternative(candidates_size() - 1);
-      }
-
-      is_numbers_expanded_ = true;
     } else {
       ExpandAlternative(candidates_size() - 1);
     }
@@ -776,7 +775,7 @@ bool Segment::Expand(size_t size) {
   // layer wants to request. Here we simply add a Katakana candidate
   // only when the number of results NbestGenerator() returns
   // is smaller than requested_candidates_size().
-  if (candidates_size() > 0 && all_expanded) {
+  if (candidates_size() > 0 && all_expanded_) {
     const string &hiragana_value = key();
     string katakana_value;
     Util::HiraganaToKatakana(key(), &katakana_value);
@@ -789,9 +788,10 @@ bool Segment::Expand(size_t size) {
       DCHECK(target_c);
       DCHECK(last_c);
       target_c->Init();
+      target_c->key = key();
       target_c->value = hiragana_value;
-      target_c->content_value = hiragana_value;
       target_c->content_key = key();
+      target_c->content_value = hiragana_value;
       target_c->cost = last_c->cost + 1;
       target_c->structure_cost = last_c->structure_cost + 1;
       target_c->lid = last_c->lid;
@@ -811,9 +811,10 @@ bool Segment::Expand(size_t size) {
       DCHECK(target_c);
       DCHECK(last_c);
       target_c->Init();
+      target_c->key = key();
       target_c->value = katakana_value;
-      target_c->content_value = katakana_value;
       target_c->content_key = key();
+      target_c->content_value = katakana_value;
       target_c->cost = last_c->cost + 1;
       target_c->structure_cost = last_c->structure_cost + 1;
       target_c->lid = last_c->lid;
@@ -829,12 +830,6 @@ bool Segment::Expand(size_t size) {
   return true;
 }
 
-bool Segment::Reset() {
-  clear_candidates();
-  nbest_generator_->Reset();
-  return true;
-}
-
 Segments::Segments(): max_history_segments_size_(0),
                       max_prediction_candidates_size_(0),
                       resized_(false),
@@ -970,6 +965,34 @@ void Segments::pop_back_segment() {
   }
 }
 
+void Segments::RemoveTailOfHistorySegments(size_t num_of_characters) {
+  size_t history_size = history_segments_size();
+  while (history_size > 0 &&
+         num_of_characters > 0) {
+    Segment *last_history_segment =
+        mutable_history_segment(history_size - 1);
+
+    const size_t length_of_last_history_segment =
+        Util::CharsLen(last_history_segment->candidate(0).value);
+
+    if (length_of_last_history_segment <= num_of_characters) {
+      num_of_characters -= length_of_last_history_segment;
+      --history_size;
+      erase_segment(history_size);
+      continue;
+    }
+
+    // Remove part of last history segment.
+    // We should change rId of the candidate too, but currently ignore it.
+    // TODO(komatsu): Treat rId properly.
+    last_history_segment->mutable_candidate(0)->value =
+        Util::SubString(last_history_segment->candidate(0).value,
+                        0,
+                        length_of_last_history_segment - num_of_characters);
+    break;
+  }
+}
+
 void Segments::clear() {
   clear_segments();
   clear_lattice();
@@ -1033,7 +1056,7 @@ void Segments::set_resized(bool resized) {
   resized_ = resized;
 }
 
-bool Segments::has_resized() const {
+bool Segments::resized() const {
   return resized_;
 }
 
@@ -1071,11 +1094,7 @@ Segments::RevertEntry *Segments::mutable_revert_entry(size_t i) {
   return &revert_entries_[i];
 }
 
-void Segments::DebugString(string *output) const {
-  if (output == NULL) {
-    return;
-  }
-  output->clear();
+string Segments::DebugString() const {
   stringstream os;
   os << "(" << endl;
   for (size_t i = 0; i < this->segments_size(); ++i) {
@@ -1087,20 +1106,23 @@ void Segments::DebugString(string *output) const {
          << seg.candidate(j).content_value
          << " cost=" << seg.candidate(j).cost
          << " scost=" << seg.candidate(j).structure_cost
+         << " wcost=" << seg.candidate(j).wcost
          << " lid=" << seg.candidate(j).lid
-         << " rid=" << seg.candidate(j).rid
-         << " prefix=" << seg.candidate(j).prefix
-         << " suffix=" << seg.candidate(j).suffix
-         << " desc=" << seg.candidate(j).description
-         << " nodes_size=" << seg.candidate(j).nodes.size();
-      for (int k = 0; k < seg.candidate(j).nodes.size(); ++k) {
-        os << "[" << seg.candidate(j).nodes[k]->lid << " "
-           << seg.candidate(j).nodes[k]->rid << "]";
+         << " rid=" << seg.candidate(j).rid;
+      if (!seg.candidate(j).prefix.empty()) {
+        os << " prefix=" << seg.candidate(j).prefix;
+      }
+      if (!seg.candidate(j).suffix.empty()) {
+        os << " suffix=" << seg.candidate(j).suffix;
+      }
+      if (!seg.candidate(j).description.empty()) {
+        os << " description=" << seg.candidate(j).description;
       }
       os << ")" << endl;
     }
   }
   os << ")" << endl;
-  *output = os.str();
+
+  return os.str();
 }
 }   // mozc
