@@ -38,6 +38,7 @@
 #include "base/util.h"
 #include "transliteration/transliteration.h"
 #include "converter/character_form_manager.h"
+#include "converter/pos_matcher.h"
 #include "converter/segments.h"
 #include "storage/lru_storage.h"
 #include "rewriter/rewriter_interface.h"
@@ -63,16 +64,6 @@ class FeatureValue {
   uint32 reserved_     : 31;  // this area is reserved for future
 };
 
-// return the "functional value" part of candidate
-string GetFunctionalValue(const Segment::Candidate &candidate) {
-  if (candidate.content_value.size() >= candidate.value.size()) {
-    return "";
-  }
-  return candidate.value.substr(candidate.content_value.size(),
-                                candidate.value.size() -
-                                candidate.content_value.size());
-}
-
 // define kLidGroup[]
 #include "rewriter/user_segment_history_rewriter_rule.h"
 
@@ -81,6 +72,24 @@ uint16 GetPosGroup(const Segment::Candidate &candidate) {
   return kLidGroup[candidate.lid];
 }
 
+bool IsPunctuation(const string &str) {
+  // return (str == "。" || str == "｡" ||
+  // str == "、" || str == "､" ||
+  // str == "，" || str == "," ||
+  // str == "．"  || str == ".");
+  return (str == "\xE3\x80\x82" || str == "\xEF\xBD\xA1" ||
+          str == "\xE3\x80\x81" || str == "\xEF\xBD\xA4" ||
+          str == "\xEF\xBC\x8C" || str == "," ||
+          str == "\xEF\xBC\x8E"  || str == ".");
+}
+
+bool IsPunctuation(const Segment &seg,
+                   const Segment::Candidate &candidate) {
+  return (POSMatcher::IsJapanesePunctuations(candidate.lid) &&
+          candidate.lid == candidate.rid &&
+          IsPunctuation(seg.key()) && IsPunctuation(candidate.value));
+}
+
 class KeyTriggerValue {
  public:
   KeyTriggerValue()
@@ -217,6 +226,44 @@ inline bool GetFeatureR(const Segments &segments, size_t i,
   return true;
 }
 
+// Feature "Left Number"
+inline bool GetFeatureLN(const Segments &segments, size_t i,
+                         const string &base_key,
+                         const string &base_value, string *value) {
+  DCHECK(value);
+  if (i < 1) {
+    return false;
+  }
+  const int j = GetDefaultCandidateIndex(segments.segment(i - 1));
+  const Segment::Candidate &candidate = segments.segment(i - 1).candidate(j);
+  if (POSMatcher::IsNumber(candidate.rid) ||
+      POSMatcher::IsKanjiNumber(candidate.rid) ||
+      Util::GetScriptType(candidate.value) == Util::NUMBER) {
+    *value = string("LN") + '\t' + base_key + '\t' + base_value;
+    return true;
+  }
+  return false;
+}
+
+// Feature "Right Number"
+inline bool GetFeatureRN(const Segments &segments, size_t i,
+                         const string &base_key,
+                         const string &base_value, string *value) {
+  DCHECK(value);
+  if (i + 1 >= segments.segments_size()) {
+    return false;
+  }
+  const int j = GetDefaultCandidateIndex(segments.segment(i + 1));
+  const Segment::Candidate &candidate = segments.segment(i + 1).candidate(j);
+  if (POSMatcher::IsNumber(candidate.lid) ||
+      POSMatcher::IsKanjiNumber(candidate.lid) ||
+      Util::GetScriptType(candidate.value) == Util::NUMBER) {
+    *value = string("RN") + '\t' + base_key + '\t' + base_value;
+    return true;
+  }
+  return false;
+}
+
 // Feature "Current"
 inline bool GetFeatureC(const Segments &segments, size_t i,
                         const string &base_key,
@@ -477,12 +524,12 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   // either top/target candidate has CONTEXT_SENSITIVE flags,
   // don't apply UNIGRAM model
   const bool context_sensitive =
-      segments.has_resized() ||
+      segments.resized() ||
       (candidate.learning_type & Segment::Candidate::CONTEXT_SENSITIVE) ||
       (segments.segment(segment_index).candidate(0).learning_type &
        Segment::Candidate::CONTEXT_SENSITIVE);
   const bool is_replaceable =
-      top_functional_value == GetFunctionalValue(candidate) &&
+      top_functional_value == candidate.functional_value() &&
       top_pos_group == GetPosGroup(candidate);
 
   DCHECK(score);
@@ -495,10 +542,11 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   uint32 last_access_time_result = 0;
   string feature_key;
 
-  const uint32 trigram_score = (segments_size == 3) ? 180 : 30;
-  const uint32 bigram_score  = (segments_size == 2) ? 60  : 10;
-  const uint32 unigram_score = (segments_size == 1) ? 36  : 6;
-  const uint32 single_score  = (segments_size == 1) ? 90  : 15;
+  const uint32 trigram_score       = (segments_size == 3) ? 180 : 30;
+  const uint32 bigram_score        = (segments_size == 2) ? 60  : 10;
+  const uint32 bigram_number_score = (segments_size == 2) ? 50  : 8;
+  const uint32 unigram_score       = (segments_size == 1) ? 36  : 6;
+  const uint32 single_score        = (segments_size == 1) ? 90  : 15;
 
   FETCH_FEATURE(GetFeatureLR, all_key, all_value, trigram_score);
   FETCH_FEATURE(GetFeatureLL, all_key, all_value, trigram_score);
@@ -506,6 +554,8 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   FETCH_FEATURE(GetFeatureL,  all_key, all_value, bigram_score);
   FETCH_FEATURE(GetFeatureR,  all_key, all_value, bigram_score);
   FETCH_FEATURE(GetFeatureS,  all_key, all_value, single_score);
+  FETCH_FEATURE(GetFeatureLN, content_key, content_value, bigram_number_score);
+  FETCH_FEATURE(GetFeatureRN, content_key, content_value, bigram_number_score);
 
   if (!context_sensitive && is_replaceable) {
     FETCH_FEATURE(GetFeatureC,  all_key, all_value, unigram_score);
@@ -521,6 +571,10 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   FETCH_FEATURE(GetFeatureL,  content_key, content_value, bigram_score / 2);
   FETCH_FEATURE(GetFeatureR,  content_key, content_value, bigram_score / 2);
   FETCH_FEATURE(GetFeatureS,  content_key, content_value, single_score / 2);
+  FETCH_FEATURE(GetFeatureLN, content_key,
+                content_value, bigram_number_score / 2);
+  FETCH_FEATURE(GetFeatureRN, content_key,
+                content_value, bigram_number_score / 2);
 
   if (!context_sensitive) {
     FETCH_FEATURE(GetFeatureC,  content_key, content_value, unigram_score / 2);
@@ -571,7 +625,14 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   }
 
   const Segment::Candidate &candidate = seg.candidate(0);
-  const bool context_sensitive = segments.has_resized() ||
+
+  // http://b/issue?id=3156109
+  // Do not remember the preference of Punctuations
+  if (IsPunctuation(seg, candidate)) {
+    return;
+  }
+
+  const bool context_sensitive = segments.resized() ||
       (candidate.learning_type & Segment::Candidate::CONTEXT_SENSITIVE);
   const string &all_value = candidate.value;
   const string &content_value = candidate.content_value;
@@ -590,8 +651,7 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   const bool is_replaceable =
       (top_index == 0) ||
       (GetPosGroup(seg.candidate(top_index)) == GetPosGroup(candidate) &&
-       GetFunctionalValue(seg.candidate(top_index)) ==
-       GetFunctionalValue(candidate));
+       seg.candidate(top_index).functional_value() == candidate.functional_value());
 
   // |feature_key| is used inside INSERT_FEATURE
   string feature_key;
@@ -601,6 +661,8 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   INSERT_FEATURE(GetFeatureRR, all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureL,  all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureR,  all_key, all_value, force_insert);
+  INSERT_FEATURE(GetFeatureLN, all_key, all_value, force_insert);
+  INSERT_FEATURE(GetFeatureRN, all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureS,  all_key, all_value, force_insert);
 
   if (!context_sensitive && is_replaceable) {
@@ -614,6 +676,8 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
     INSERT_FEATURE(GetFeatureRR, content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureL,  content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureR,  content_key, content_value, force_insert);
+    INSERT_FEATURE(GetFeatureLN, content_key, content_value, force_insert);
+    INSERT_FEATURE(GetFeatureRN, content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureS,  content_key, content_value, force_insert);
     if (!context_sensitive) {
       INSERT_FEATURE(GetFeatureC, content_key, content_value, force_insert);
@@ -780,11 +844,11 @@ bool UserSegmentHistoryRewriter::RewriteNumber(Segment *segment) const {
            != Segment::Candidate::NUMBER_SEPARATED_ARABIC_HALFWIDTH)) {
         last_access_time--;
       }
+      scores.resize(scores.size() + 1);
+      scores.back().score = score;
+      scores.back().last_access_time = last_access_time;
+      scores.back().candidate = segment->mutable_candidate(j);
     }
-    scores.resize(scores.size() + 1);
-    scores.back().score = score;
-    scores.back().last_access_time = last_access_time;
-    scores.back().candidate = segment->mutable_candidate(j);
   }
 
   if (scores.empty()) {
@@ -826,6 +890,10 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
       continue;
     }
 
+    if (IsPunctuation(*segment, segment->candidate(0))) {
+      continue;
+    }
+
     if (IsNumberSegment(*segment)) {
       modified |= RewriteNumber(segment);
       continue;
@@ -846,7 +914,7 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
     }
 
     const string top_functional_value =
-        GetFunctionalValue(segment->candidate(0));
+        segment->candidate(0).functional_value();
     const uint16 top_pos_group = GetPosGroup(segment->candidate(0));
 
     // for each all candidates expanded
