@@ -42,6 +42,7 @@
 #include "base/version.h"
 #include "composer/composer.h"
 #include "composer/table.h"
+#include "rewriter/calculator/calculator_interface.h"
 #include "session/commands.pb.h"
 #include "session/config_handler.h"
 #include "session/config.pb.h"
@@ -240,6 +241,42 @@ void InitTransformTable(const config::Config &config, TransformTable *table) {
   }
 }
 
+// Logic of nested calculation
+// Returns the number of characters to expand preedit to left.
+size_t GetCompositionExpansionForCalculator(const string &preceding_text,
+                                            const string &composition,
+                                            string *expanded_characters) {
+  DCHECK(expanded_characters != NULL);
+
+  // Return 0, if last character is neither "=" nor "＝".
+  if (composition.empty() ||
+      !(Util::EndsWith(composition, "=") ||
+        Util::EndsWith(composition, "\xEF\xBC\x9D"))) {
+    return 0;
+  }
+
+  const CalculatorInterface *calculator = CalculatorFactory::GetCalculator();
+
+  string result;
+  const size_t preceding_length = Util::CharsLen(preceding_text);
+  size_t expansion = preceding_length;
+  while (expansion > 0) {
+    const string part_of_preceding =
+        Util::SubString(preceding_text,
+                        preceding_length - expansion,
+                        expansion);
+    const string key = part_of_preceding + composition;
+    // Skip if the first character is space.
+    if (!Util::StartsWith(key, " ") &&
+        calculator->CalculateString(key, &result)) {
+      *expanded_characters = part_of_preceding;
+      break;
+    }
+    --expansion;
+  }
+  return expansion;
+}
+
 // Set input mode if the current input mode is not the given mode.
 void SwitchInputMode(const transliteration::TransliterationType mode,
                      composer::Composer *composer) {
@@ -270,9 +307,22 @@ Session::Session(const composer::Table *table,
 
 Session::~Session() {}
 
+void Session::SetSessionState(const SessionState::Type state) {
+  state_ = state;
+  if (state == SessionState::DIRECT ||
+      state == SessionState::PRECOMPOSITION) {
+    composer_->Reset();
+  } else if (state == SessionState::CONVERSION) {
+    composer_->ResetInputMode();
+  }
+  // else if (state == SessionState::COMPOSITION) {
+  //   Do nothing.
+  // }
+}
+
 void Session::EnsureIMEIsOn() {
   if (state_ == SessionState::DIRECT) {
-    state_ = SessionState::PRECOMPOSITION;
+    SetSessionState(SessionState::PRECOMPOSITION);
   }
 }
 
@@ -282,10 +332,7 @@ bool Session::SendCommand(commands::Command *command) {
   if (!command->input().has_command()) {
     return false;
   }
-  if (command->input().has_key()) {
-    TransformKeyEvent(transform_table_,
-                      command->mutable_input()->mutable_key());
-  }
+  TransformInput(command->mutable_input());
   const commands::SessionCommand &session_command = command->input().command();
 
   if (session_command.type() == commands::SessionCommand::SWITCH_INPUT_MODE) {
@@ -345,10 +392,7 @@ bool Session::TestSendKey(commands::Command *command) {
   if (!keymap_) {
     return false;
   }
-  if (command->input().has_key()) {
-    TransformKeyEvent(transform_table_,
-                      command->mutable_input()->mutable_key());
-  }
+  TransformInput(command->mutable_input());
 
   if (state_ == SessionState::NONE) {
     // This must be an error.
@@ -418,10 +462,7 @@ bool Session::SendKey(commands::Command *command) {
   if (!keymap_) {
     return false;
   }
-  if (command->input().has_key()) {
-    TransformKeyEvent(transform_table_,
-                      command->mutable_input()->mutable_key());
-  }
+  TransformInput(command->mutable_input());
 
   bool result = false;
   switch (state_) {
@@ -454,6 +495,11 @@ bool Session::SendKeyDirectInputState(commands::Command *command) {
   if (!keymap_->GetCommandDirect(command->input().key(), &key_command)) {
     return EchoBack(command);
   }
+  string command_name;
+  if (keymap_->GetNameFromCommandDirect(key_command, &command_name)) {
+    const string name = "Direct_" + command_name;
+    command->mutable_output()->set_performed_command(name);
+  }
   switch (key_command) {
     case keymap::DirectInputState::IME_ON:
       return IMEOn(command);
@@ -483,6 +529,11 @@ bool Session::SendKeyPrecompositionState(commands::Command *command) {
                                          &key_command)) {
     return EchoBack(command);
   }
+  string command_name;
+  if (keymap_->GetNameFromCommandPrecomposition(key_command, &command_name)) {
+    const string name = "Precomposition_" + command_name;
+    command->mutable_output()->set_performed_command(name);
+  }
   switch (key_command) {
     case keymap::PrecompositionState::INSERT_CHARACTER:
       return InsertCharacter(command);
@@ -534,7 +585,11 @@ bool Session::SendKeyCompositionState(commands::Command *command) {
   if (!result) {
     return DoNothing(command);
   }
-
+  string command_name;
+  if (keymap_->GetNameFromCommandComposition(key_command, &command_name)) {
+    const string name = "Composition_" + command_name;
+    command->mutable_output()->set_performed_command(name);
+  }
   switch (key_command) {
     case keymap::CompositionState::INSERT_CHARACTER:
       return InsertCharacter(command);
@@ -663,7 +718,11 @@ bool Session::SendKeyConversionState(commands::Command *command) {
   if (!result) {
     return DoNothing(command);
   }
-
+  string command_name;
+  if (keymap_->GetNameFromCommandConversion(key_command, &command_name)) {
+    const string name = "Conversion_" + command_name;
+    command->mutable_output()->set_performed_command(name);
+  }
   switch (key_command) {
     case keymap::ConversionState::INSERT_CHARACTER:
       return InsertCharacter(command);
@@ -792,17 +851,24 @@ bool Session::SendKeyConversionState(commands::Command *command) {
 }
 
 bool Session::UpdatePreferences(commands::Command *command) {
-  if (command == NULL ||
-      !command->input().has_config()) {
+  if (command == NULL) {
     return false;
   }
-  converter_->UpdateConfig(command->input().config());
-  return true;
+  bool result = false;
+  if (command->input().has_config()) {
+    converter_->UpdateConfig(command->input().config());
+    result = true;
+  }
+  if (command->input().has_capability()) {
+    client_capability_ = command->input().capability();
+    result = true;
+  }
+  return result;
 }
 
 bool Session::IMEOn(commands::Command *command) {
   command->mutable_output()->set_consumed(true);
-  state_ = SessionState::PRECOMPOSITION;
+  SetSessionState(SessionState::PRECOMPOSITION);
   const commands::KeyEvent &key = command->input().key();
   if (key.has_mode()) {
     // Ime on with specified mode.
@@ -839,7 +905,7 @@ bool Session::IMEOff(commands::Command *command) {
   //  EditCancel(command);
   Commit(command);
 
-  state_ = SessionState::DIRECT;
+  SetSessionState(SessionState::DIRECT);
   OutputMode(command);
   return true;
 }
@@ -888,12 +954,7 @@ bool Session::Revert(commands::Command *command) {
     converter_->Cancel();
   }
 
-  // The first guard clause in this function guarantees that the current state
-  // is COMPOSITION or CONVERSION here.
-  // In either case, we should reset the composer.
-  composer_->Reset();
-
-  state_ = SessionState::PRECOMPOSITION;
+  SetSessionState(SessionState::PRECOMPOSITION);
   OutputMode(command);
   return true;
 }
@@ -910,7 +971,7 @@ bool Session::GetStatus(commands::Command *command) {
   return true;
 }
 
-bool Session::SelectCandidate(commands::Command *command) {
+bool Session::SelectCandidateInternal(commands::Command *command) {
   // If the current state is not conversion or composition, the
   // candidate window should not be shown.  (On composition, the
   // window is able to be shown as a suggestion window).
@@ -925,14 +986,21 @@ bool Session::SelectCandidate(commands::Command *command) {
 
   command->mutable_output()->set_consumed(true);
   converter_->CandidateMoveToId(command->input().command().id());
-  state_ = SessionState::CONVERSION;
+  SetSessionState(SessionState::CONVERSION);
 
+  return true;
+}
+
+bool Session::SelectCandidate(commands::Command *command) {
+  if (!SelectCandidateInternal(command)) {
+    return false;
+  }
   Output(command);
   return true;
 }
 
 bool Session::HighlightCandidate(commands::Command *command) {
-  if (!SelectCandidate(command)) {
+  if (!SelectCandidateInternal(command)) {
     return false;
   }
   converter_->SetCandidateListVisible(true);
@@ -953,6 +1021,19 @@ bool Session::MaybeSelectCandidate(commands::Command *command) {
 }
 
 
+void Session::set_client_capability(const commands::Capability &capability) {
+  client_capability_.CopyFrom(capability);
+}
+
+void Session::set_application_info(const commands::ApplicationInfo
+                                   &application_info) {
+  application_info_.CopyFrom(application_info);
+}
+
+const commands::ApplicationInfo &Session::application_info() const {
+  return application_info_;
+}
+
 bool Session::InsertCharacter(commands::Command *command) {
   // If the input_style is DIRECT_INPUT, KeyEvent is not consumed and
   // done echo back.  It works only when key_string is equal to
@@ -987,7 +1068,7 @@ bool Session::InsertCharacter(commands::Command *command) {
       result->mutable_key()->append(command->input().key().key_string());
       // Append a character representing key_code.
       result->mutable_value()->append(1, command->input().key().key_code());
-      state_ = SessionState::PRECOMPOSITION;
+      SetSessionState(SessionState::PRECOMPOSITION);
       return true;
     }
   }
@@ -999,14 +1080,20 @@ bool Session::InsertCharacter(commands::Command *command) {
 
   composer_->InsertCharacterKeyEvent(command->input().key());
 
-  state_ = SessionState::COMPOSITION;
+  ExpandCompositionForCalculator(command);
+
+  SetSessionState(SessionState::COMPOSITION);
+  if (CanStartAutoConversion(command->input().key())) {
+    return Convert(command);
+  }
+
   if (converter_->Suggest(composer_.get())) {
     DCHECK(converter_->IsActive());
     Output(command);
     return true;
   }
-
   OutputComposition(command);
+
   return true;
 }
 
@@ -1128,8 +1215,7 @@ bool Session::InsertSpaceFullWidth(commands::Command *command) {
 
 bool Session::EditCancel(commands::Command *command) {
   command->mutable_output()->set_consumed(true);
-  composer_->Reset();
-  state_ = SessionState::PRECOMPOSITION;
+  SetSessionState(SessionState::PRECOMPOSITION);
   OutputMode(command);
   return true;
 }
@@ -1145,8 +1231,7 @@ bool Session::Commit(commands::Command *command) {
   } else {  // SessionState::CONVERSION
     converter_->Commit();
   }
-  composer_->Reset();
-  state_ = SessionState::PRECOMPOSITION;
+  SetSessionState(SessionState::PRECOMPOSITION);
   Output(command);
   return true;
 }
@@ -1160,8 +1245,7 @@ bool Session::CommitFirstSuggestion(commands::Command *command) {
   const int kFirstIndex = 0;
   converter_->CommitSuggestion(kFirstIndex);
 
-  composer_->Reset();
-  state_ = SessionState::PRECOMPOSITION;
+  SetSessionState(SessionState::PRECOMPOSITION);
   Output(command);
   return true;
 }
@@ -1176,8 +1260,7 @@ bool Session::CommitSegment(commands::Command *command) {
   if (!converter_->IsActive()) {
     // If the converter is not active (ie. the segment size was one.),
     // the state should be switched to precomposition.
-    composer_->Reset();
-    state_ = SessionState::PRECOMPOSITION;
+    SetSessionState(SessionState::PRECOMPOSITION);
   }
   Output(command);
   return true;
@@ -1194,7 +1277,7 @@ bool Session::ConvertToTransliteration(
   if (!converter_->ConvertToTransliteration(composer_.get(), type)) {
     return false;
   }
-  state_ = SessionState::CONVERSION;
+  SetSessionState(SessionState::CONVERSION);
   Output(command);
   return true;
 }
@@ -1228,7 +1311,7 @@ bool Session::SwitchKanaType(commands::Command *command) {
   if (!converter_->SwitchKanaType(composer_.get())) {
     return false;
   }
-  state_ = SessionState::CONVERSION;
+  SetSessionState(SessionState::CONVERSION);
   Output(command);
   return true;
 }
@@ -1344,7 +1427,7 @@ bool Session::ConvertToHalfWidth(commands::Command *command) {
   if (!converter_->ConvertToHalfWidth(composer_.get())) {
     return false;
   }
-  state_ = SessionState::CONVERSION;
+  SetSessionState(SessionState::CONVERSION);
   Output(command);
   return true;
 }
@@ -1416,7 +1499,7 @@ bool Session::Convert(commands::Command *command) {
     return true;
   }
 
-  state_ = SessionState::CONVERSION;
+  SetSessionState(SessionState::CONVERSION);
   Output(command);
   return true;
 }
@@ -1433,7 +1516,7 @@ bool Session::ConvertWithoutHistory(commands::Command *command) {
     return true;
   }
 
-  state_ = SessionState::CONVERSION;
+  SetSessionState(SessionState::CONVERSION);
   Output(command);
   return true;
 }
@@ -1490,9 +1573,7 @@ bool Session::Delete(commands::Command *command) {
   command->mutable_output()->set_consumed(true);
   composer_->Delete();
   if (composer_->Empty()) {
-    // Input/output modes remain.
-    composer_->EditErase();
-    state_ = SessionState::PRECOMPOSITION;
+    SetSessionState(SessionState::PRECOMPOSITION);
     OutputMode(command);
   } else if (converter_->Suggest(composer_.get())) {
     DCHECK(converter_->IsActive());
@@ -1508,9 +1589,7 @@ bool Session::Backspace(commands::Command *command) {
   command->mutable_output()->set_consumed(true);
   composer_->Backspace();
   if (composer_->Empty()) {
-    // Input/output modes remain.
-    composer_->EditErase();
-    state_ = SessionState::PRECOMPOSITION;
+    SetSessionState(SessionState::PRECOMPOSITION);
     OutputMode(command);
   } else if (converter_->Suggest(composer_.get())) {
     DCHECK(converter_->IsActive());
@@ -1617,7 +1696,7 @@ bool Session::ConvertPrevPage(commands::Command *command) {
 bool Session::ConvertCancel(commands::Command *command) {
   command->mutable_output()->set_consumed(true);
 
-  state_ = SessionState::COMPOSITION;
+  SetSessionState(SessionState::COMPOSITION);
   converter_->Cancel();
   if (converter_->Suggest(composer_.get())) {
     DCHECK(converter_->IsActive());
@@ -1636,7 +1715,7 @@ bool Session::PredictAndConvert(commands::Command *command) {
 
   command->mutable_output()->set_consumed(true);
   if (converter_->Predict(composer_.get())) {
-    state_ = SessionState::CONVERSION;
+    SetSessionState(SessionState::CONVERSION);
     Output(command);
   } else {
     OutputComposition(command);
@@ -1666,19 +1745,36 @@ void Session::Output(commands::Command *command) {
 }
 
 void Session::OutputMode(commands::Command *command) const {
+  commands::CompositionMode mode = commands::HIRAGANA;
+  switch (composer_->GetInputMode()) {
+  case transliteration::HIRAGANA:
+    mode = commands::HIRAGANA;
+    break;
+  case transliteration::FULL_KATAKANA:
+    mode = commands::FULL_KATAKANA;
+    break;
+  case transliteration::HALF_KATAKANA:
+    mode = commands::HALF_KATAKANA;
+    break;
+  case transliteration::FULL_ASCII:
+    mode = commands::FULL_ASCII;
+    break;
+  case transliteration::HALF_ASCII:
+    mode = commands::HALF_ASCII;
+    break;
+  default:
+    LOG(ERROR) << "Unknown input mode: " << composer_->GetInputMode();
+    // use HIRAGANA as a default.
+  }
+
   if (state_ == SessionState::DIRECT) {
     command->mutable_output()->set_mode(commands::DIRECT);
-  } else if (composer_->GetInputMode() == transliteration::HIRAGANA) {
-    command->mutable_output()->set_mode(commands::HIRAGANA);
-  } else if (composer_->GetInputMode() == transliteration::FULL_KATAKANA) {
-    command->mutable_output()->set_mode(commands::FULL_KATAKANA);
-  } else if (composer_->GetInputMode() == transliteration::HALF_KATAKANA) {
-    command->mutable_output()->set_mode(commands::HALF_KATAKANA);
-  } else if (composer_->GetInputMode() == transliteration::FULL_ASCII) {
-    command->mutable_output()->set_mode(commands::FULL_ASCII);
-  } else if (composer_->GetInputMode() == transliteration::HALF_ASCII) {
-    command->mutable_output()->set_mode(commands::HALF_ASCII);
+    command->mutable_output()->mutable_status()->set_activated(false);
+  } else {
+    command->mutable_output()->set_mode(mode);
+    command->mutable_output()->mutable_status()->set_activated(true);
   }
+  command->mutable_output()->mutable_status()->set_mode(mode);
 }
 
 void Session::OutputComposition(commands::Command *command) const {
@@ -1693,7 +1789,170 @@ void Session::OutputKey(commands::Command *command) const {
   key->CopyFrom(command->input().key());
 }
 
+namespace {
+// return
+// ((key_code == static_cast<uint32>('.') ||
+//       key_string == "." || key_string == "．" ||
+//   key_string == "。" || key_string == "｡") &&
+//  (config.auto_conversion_key() &
+//   config::Config::AUTO_CONVERSION_KUTEN)) ||
+// ((key_code == static_cast<uint32>(',') ||
+//       key_string == "," || key_string == "，" ||
+//   key_string == "、" || key_string == "､") &&
+//  (config.auto_conversion_key() &
+//   config::Config::AUTO_CONVERSION_TOUTEN)) ||
+// ((key_code == static_cast<uint32>('?') ||
+//   key_string == "?" || key_string == "？") &&
+//  (config.auto_conversion_key() &
+//   config::Config::AUTO_CONVERSION_QUESTION_MARK)) ||
+// ((key_code == static_cast<uint32>('!') ||
+//   key_string == "!" || key_string == "！") &&
+//  (config.auto_conversion_key() &
+//   config::Config::AUTO_CONVERSION_EXCLAMATION_MARK));
+bool IsValidKey(const config::Config &config,
+                const uint32 key_code, const string &key_string) {
+  return
+      (((key_code == static_cast<uint32>('.') && key_string.empty()) ||
+        key_string == "." || key_string == "\xEF\xBC\x8E" ||
+        key_string == "\xE3\x80\x82" || key_string == "\xEF\xBD\xA1") &&
+       (config.auto_conversion_key() &
+        config::Config::AUTO_CONVERSION_KUTEN)) ||
+      (((key_code == static_cast<uint32>(',') && key_string.empty()) ||
+        key_string == "," || key_string == "\xEF\xBC\x8C" ||
+        key_string == "\xE3\x80\x81" || key_string == "\xEF\xBD\xA4") &&
+       (config.auto_conversion_key() &
+        config::Config::AUTO_CONVERSION_TOUTEN)) ||
+      (((key_code == static_cast<uint32>('?') && key_string.empty()) ||
+        key_string == "?" || key_string == "\xEF\xBC\x9F") &&
+       (config.auto_conversion_key() &
+        config::Config::AUTO_CONVERSION_QUESTION_MARK)) ||
+      (((key_code == static_cast<uint32>('!') && key_string.empty()) ||
+        key_string == "!" || key_string == "\xEF\xBC\x81") &&
+       (config.auto_conversion_key() &
+        config::Config::AUTO_CONVERSION_EXCLAMATION_MARK));
+}
+}  // namespace
+
+bool Session::CanStartAutoConversion(
+    const commands::KeyEvent &key_event) const {
+  if (!GET_CONFIG(use_auto_conversion)) {
+    return false;
+  }
+
+  // Disable if the input comes from non-standard user keyboards, like numpad.
+  // http://b/issue?id=2932067
+  if (key_event.input_style() != commands::KeyEvent::FOLLOW_MODE) {
+    return false;
+  }
+
+  // This is a tentative workaround for the bug http://b/issue?id=2932028
+  // When user types <Shift Down>O<Shift Up>racle<Shift Down>!<Shift Up>,
+  // The final "!" must be half-width, however, due to the limitation
+  // of converter interface, we don't have a good way to change it halfwidth, as
+  // the default preference of "!" is fullwidth. Basically, the converter is
+  // not composition-mode-aware.
+  // We simply disable the auto conversion feature if the mode is ASCII.
+  // We conclude that disabling this feature is better in this situation.
+  // TODO(taku): fix the behavior. Converter module needs to be fixed.
+  if (key_event.mode() == commands::HALF_ASCII ||
+      key_event.mode() == commands::FULL_ASCII) {
+    return false;
+  }
+
+  const config::Config &config = config::ConfigHandler::GetConfig();
+  const uint32 key_code = key_event.key_code();
+  const string &key_string = key_event.key_string();
+
+  // first, check raw user key encoded in |key_code| and |key_string|.
+  // At this moment, we don't check the preedit string.
+  // We'd like to return this function as early as possible, since
+  // auto_conversion feature isn't a default feature and will not
+  // be activated often.
+  // We can suppose that this function returns false here in almost all case,
+  // even when auto_convesion is true.
+  if (!IsValidKey(config, key_code, key_string)) {
+    return false;
+  }
+
+  // now evaluate preedit string and preedit length.
+  const size_t length = composer_->GetLength();
+  if (length <= 1) {
+    return false;
+  }
+
+  string preedit;
+  DCHECK(composer_.get());
+  composer_->GetStringForPreedit(&preedit);
+  const string last_char = Util::SubString(preedit, length - 1, 1);
+  if (last_char.empty()) {
+    return false;
+  }
+
+  // Check last character as user may change romaji table,
+  // For instance, if user assigns "." as "foo", we don't
+  // want to invoke auto_conversion.
+  if (!IsValidKey(config, key_code, last_char)) {
+    return false;
+  }
+
+  // check the previous character of last_character.
+  // when |last_prev_char| is number, we don't invoke auto_conversion
+  // if the same invoke key is repeated, do not conversion.
+  // http://b/issue?id=2932118
+  const string last_prev_char = Util::SubString(preedit, length - 2, 1);
+  if (last_prev_char.empty() || last_prev_char == last_char ||
+      Util::NUMBER == Util::GetScriptType(last_prev_char)) {
+    return false;
+  }
+
+  return true;
+}
+
 void Session::UpdateTime() {
   last_command_time_ = Util::GetTime();
 }
+
+void Session::TransformInput(commands::Input *input) {
+  if (input->has_key()) {
+    TransformKeyEvent(transform_table_, input->mutable_key());
+  }
+  converter_->FillContext(input->mutable_context());
+}
+
+void Session::ExpandCompositionForCalculator(commands::Command *command) {
+  if ((client_capability_.text_deletion() &
+           commands::Capability::DELETE_PRECEDING_TEXT) == 0) {
+    return;
+  }
+  if (!command->input().has_context()) {
+    return;
+  }
+
+  // Expand composition if expanded composition makes an expression of
+  // Calculator.
+  // E.g. if preceding text is "あいう１" and composition is "+1=", then
+  // composition is expanded to "１+1=".
+  string preedit, expanded_characters;
+  composer_->GetStringForPreedit(&preedit);
+  const size_t expansion_length =
+      GetCompositionExpansionForCalculator(
+          command->input().context().preceding_text(),
+          preedit,
+          &expanded_characters);
+
+  if (expansion_length == 0) {
+    return;
+  }
+
+  composer_->InsertCharacterPreeditAt(0, expanded_characters);
+
+  commands::DeletionRange *range =
+      command->mutable_output()->mutable_deletion_range();
+  range->set_offset(-expansion_length);
+  range->set_length(expansion_length);
+
+  // Delete part of history segments, because corresponding surrounding
+  // text will be removed by client.
+  converter_->RemoveTailOfHistorySegments(expansion_length);
+}
 }  // namespace mozc
